﻿using System;
using System.IO;
using System.Net;
using System.Text;
using Newtonsoft.Json;

namespace Nautilus
{
    class Program
    {
        private static HttpListener _listener;

        private static void Main()
        {
            Console.WriteLine("IDScan.net ID Parsing Server");
            _listener = new HttpListener();
            _listener.Prefixes.Add("http://localhost:9999/");
            _listener.AuthenticationSchemes = AuthenticationSchemes.Anonymous;
            _listener.Start();
            while (true)
            {
                ProcessRequest();
            }
        }

        private static void ProcessRequest()
        {
            var result = _listener.BeginGetContext(ListenerCallback, _listener);
            result.AsyncWaitHandle.WaitOne();
        }

        private static void ListenerCallback(IAsyncResult result)
        {
            var context = _listener.EndGetContext(result);
            context.Response.StatusCode = 200;
            context.Response.StatusDescription = "OK";

            var request = context.Request;
            using (var body = request.InputStream)
            using (var reader = new StreamReader(body, request.ContentEncoding))
            {
                var scannedText = Encoding.UTF8.GetString(Convert.FromBase64String(reader.ReadToEnd()));
                try
                {
                    var lic = DriverLicense.ParseText(scannedText);
                    context.Response.Headers["Result"] = lic != null ? "OK" : "NOK";
                    if (lic != null)
                    {
                        context.Response.ContentEncoding = Encoding.UTF8;
                        context.Response.ContentType = "application/json";
                        using (var stream = context.Response.OutputStream)
                        {
                            var json = JsonConvert.SerializeObject(lic, Formatting.Indented);
                            var data = Encoding.UTF8.GetBytes(json);
                            stream.Write(data, 0, data.Length);
                        }
                    }
                }
                catch (Exception ex)
                {
                    context.Response.Headers["Result"] = "Exception";
                    context.Response.Headers["Exception"] = ex.Message;
                }
                context.Response.Close();
            }
        }
    }
}
